##########################################################################################################################################
### Identification of potential biomarkers for anti-PD-1 therapy in melanoma by weighted correlation network analysis, Mar 25th, 2020.
### Author: Xuanyi Wang, Zixuan Chai 1, Yinghong Li, Fei Long, Youjin Hao, Guizhi Pan, Mingwei Liu, and Bo Li.
##########################################################################################################################################

### ***************************************************************************************
### Construction of a random forest model for predicting anti-PD-1 therapeutic response
### ***************************************************************************************

### ****************************************************************************
### code chunk number 01: Setting the work directory and preparing data.
### ****************************************************************************

### ----------------------1.1 Setting the work dir.------------------------- ### 

rm(list = ls())
options(stringsAsFactors = F)
setwd(':/')

### --------------------------1.2 load packages.---------------------------- ### 

library(randomForest)
library(ROCR)
library(genefilter)
library(Hmisc)
library(caret)

### -------------------1.3 Reading gene expression data.-------------------- ### 

### 1.3.1 Input the gene expression matrix. 

train_test_exprSet <- read.table("GSE78220_FPKM_Matrix.txt", sep='\t', row.names=1, header=T, 
                      quote="", comment="", check.names=F)

validation_exprSet <- read.table("GSE93157_log2_Matrix.txt", sep='\t', row.names=1, header=T, 
                                 quote="", comment="", check.names=F)

### 1.3.2 Input the gene expression meta data.

train_test_meta <- read.table("GSE78220_meta.txt", sep='\t', header=T, 
                   check.names=FALSE, comment='',quote="")

validation_meta <- read.table("GSE93157_meta.txt", sep='\t', header=T, 
                   check.names=FALSE, comment='',quote="")

### ---------------1.4 Partition training set, test set and validation set.----------------- ### 

set.seed(12345679)
train_test_sam<- createDataPartition(train_test_meta$event, p = .5,list = FALSE)
train <- train_test_exprSet[,train_test_sam]
test <- train_test_exprSet[,-train_test_sam]
train_meta <- train_test_meta[train_test_sam,]
test_meta <- train_test_meta[-train_test_sam,]

set.seed(12345679)
validation_sam<- createDataPartition(validation_meta$group, p = .5,list = FALSE)
validation <- validation_exprSet[,validation_sam]
validation_meta <- validation_meta[validation_sam,]

### ---------------1.5 Partition training set, test set and validation set.----------------- ### 

exprSet_6genes_train=(train[c('CD8A','IRF1','IRF8','JAK2','SELL','STAT5B'),])
Preprocessing_train = t(log2(exprSet_6genes_train+1))

exprSet_6genes_test=(test[c('CD8A','IRF1','IRF8','JAK2','SELL','STAT5B'),])
Preprocessing_test = t(log2(exprSet_6genes_test+1))

exprSet_6genes_validation = (validation[c('CD8A','IRF1','IRF8','JAK2','SELL','STAT5B'),])
Preprocessing_validation = t(log2(exprSet_6genes_validation+1))

dat_train=cbind(train_meta[,3],Preprocessing_train)
colnames(dat_train)[1]=colnames(train_meta)[3]
dat_train=as.data.frame(dat_train)
dat_train$group=factor(dat_train$group)

dat_test=cbind(test_meta[,3],Preprocessing_test)
colnames(dat_test)[1]=colnames(test_meta)[3]
dat_test=as.data.frame(dat_test)
dat_test$group=factor(dat_test$group)

dat_validation=cbind(validation_meta[,2],Preprocessing_validation)
colnames(dat_validation)[1]=colnames(validation_meta)[2]
dat_validation=as.data.frame(dat_validation)
dat_validation$group=factor(dat_validation$group)

### ****************************************************************************
### code chunk number 02: Building random forest model.
### ****************************************************************************

### ----2.1 Find the optimal number of variables in the specified node for binary tree.---- ### 

#Finding the best parameter of mtry.
n=ncol(dat_train) #Counting the number of independent variables in the dataset.
rate=1 #Setting the initial value of model error rate vector.
for(i in 1:(n-1)){
  set.seed(1234)
  rf_train<-randomForest(as.factor(dat_train$group)~.,data=dat_train,mtry=i,ntree=10000)
  rate[i]<-mean(rf_train$err.rate)  #Calculating the mean value of model error rate based on OOB data.
  print(rf_train)    
}
rate  #Showing the mean value of misjudgment rate of all models.
plot(rate)

### ----------2.2 Finding the optimal number of decision trees in a random forest.--------- ### 

#Finding the best parameter of nTree.
set.seed(100)
rf_train_tree<-randomForest(as.factor(dat_train$group)~.,data=dat_train,mtry=3,ntree=10000)
plot(rf_train_tree)  #Drawing the relationship between model error and decision tree quantity.

### ----------------2.3 Using training set to construct random forest model.--------------- ### 

rf_output = randomForest(group ~ ., dat_train, ntree=2000, 
                         nPerm=10, mtry=3, proximity=TRUE, importance=TRUE)  

print(rf_output)  #Displaying brief information of random forest model.
hist(treesize(rf_output))  #Showing the number of nodes of each decision tree in the random forest model.

### -------------------2.4 Useing training set model to predict test set and validation set.------------------ ### 

### 2.4.1 predicting test set and validation set. 

obs_test = dat_test$group  #Observations of test set metadata.

obs_validation = dat_validation$group #Observations of validation set metadata.


test_rf.prob <- predict(rf_output, dat_test)

validation_rf.prob <- predict(rf_output, dat_validation)

### 2.4.2 Constructing the matrix of observation value and prediction value.

test_re=cbind(obs_test ,test_rf.prob)
test_re=as.data.frame(test_re)
colnames(test_re)=c('event','prob')
test_re$event=as.factor(test_re$event)

validation_re=cbind(obs_validation ,validation_rf.prob)
validation_re=as.data.frame(validation_re)
colnames(validation_re)=c('event','prob')
validation_re$event=as.factor(validation_re$event)

### ****************************************************************************
### code chunk number 03: Evaluating random forest model.
### ****************************************************************************

### -------------------3.1 Calculating the AUC of random forest model.------------------ ### 

test_pred <- prediction(test_re[,2], test_re[,1])
test_perf <- performance(test_pred,"tpr","fpr")
test_auc= performance(test_pred,"auc")@y.values[[1]]
test_auc

validation_pred <- prediction(validation_re[,2], validation_re[,1])
validation_perf <- performance(validation_pred,"tpr","fpr")
validation_auc = performance(validation_pred,"auc")@y.values[[1]]
validation_auc

table(observed=validation_meta$group, predicted=validation_rf.prob) 
table(observed=test_meta$group, predicted=test_rf.prob) 

### --------------------3.2 Drawing ROC curve of random forest model.------------------- ### 

test_tpr = performance(test_pred,"tpr")@y.values[[1]]
dat_test_roc = data.frame(tpr = test_perf@y.values[[1]],
                 fpr = test_perf@x.values[[1]])

ggplot() + 
  geom_line(data = dat_test_roc,aes(x = fpr, y = tpr),color = "blue") + 
  geom_line(aes(x=c(0,1),y=c(0,1)),color = "grey")+
  theme_bw()+
  annotate("text",x = .75, y = .25,
           label = paste("AUC = ",round(test_auc,2)),color = "blue")+
  scale_x_continuous(name  = "fpr")+
  scale_y_continuous(name = "tpr")


validation_tpr = performance(validation_pred,"tpr")@y.values[[1]]
dat_validation_roc = data.frame(tpr = validation_perf@y.values[[1]],
                     fpr = validation_perf@x.values[[1]])

ggplot() + 
  geom_line(data = dat_validation_roc,aes(x = fpr, y = tpr),color = "blue") + 
  geom_line(aes(x=c(0,1),y=c(0,1)),color = "grey")+
  theme_bw()+
  annotate("text",x = .75, y = .25,
           label = paste("AUC = ",round(validation_auc,2)),color = "blue")+
  scale_x_continuous(name  = "fpr")+
  scale_y_continuous(name = "tpr")

### ------------------------3.3 Deriving random forest model data.------------------------ ### 

save(rf_output,file = "rf_output.Rdata")






